@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param database Groups of individuals to consider for the export
 #' @param gen Quick-insert for database (vector of all generations to export)
 #' @param cohorts Quick-insert for database (vector of names of cohorts to export)
-#' @param bvrow Which traits to display (for multiple traits separte plots (par(mfrow)))
+#' @param bvrow Which traits to display (for multiple traits separate plots (par(mfrow)))
 #' @param json If TRUE extract which cohorts to plot according to the json-file used in json.simulation
 #' @param ylim Set this to fix the y-axis of the plot
 #' @param fix_mfrow Set TRUE to not use mfrow - use for custom plots
