@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,31 +21,45 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #' Derive genotypes of selected individuals
 #'
-#' Function to devide genotypes of selected individuals
+#' Function to derive genotypes of selected individuals
 #' @param population Population list
 #' @param database Groups of individuals to consider for the export
 #' @param gen Quick-insert for database (vector of all generations to export)
 #' @param cohorts Quick-insert for database (vector of names of cohorts to export)
-#' @param chromosomen Beschraenkung des Genotypen auf bestimmte Chromosomen (default: 1)
+#' @param id Individual IDs to search/collect in the database
+#' @param chromosome Limit the genotype output to a selected chromosome (default: "all")
 #' @param export.alleles If TRUE export underlying alleles instead of just 012
 #' @param non.genotyped.as.missing Set to TRUE to replace non-genotyped markers with NA
-#' @param use.id Set to TRUE to use MoBPS ids instead of Sex_Nr_Gen based names (default: FALSE)
+#' @param use.id Set to TRUE to use MoBPS ids instead of Sex_Nr_Gen based names (default: TRUE)
+#' @param array Use only markers available on the array
+#' @param remove.missing Remove markers not genotyped in any individual from the export
 #' @examples
 #' data(ex_pop)
 #' geno <- get.geno(ex_pop, gen=2)
 #' @return Genotype data for in gen/database/cohorts selected individuals
 #' @export
 
-get.geno <- function(population, database=NULL, gen=NULL, cohorts=NULL, chromosomen="all", export.alleles=FALSE,
-                     non.genotyped.as.missing=FALSE, use.id=FALSE){
+get.geno <- function(population, database=NULL, gen=NULL, cohorts=NULL, id = NULL, chromosome="all", export.alleles=FALSE,
+                      non.genotyped.as.missing=FALSE, use.id=TRUE, array = NULL, remove.missing = TRUE){
 
-  if(length(chromosomen)==1 && chromosomen=="all"){
+  if(length(chromosome)==1 && chromosome=="all"){
     subsetting <- FALSE
-    chromosomen <- 1:length(population$info$snp)
+    chromosome <- 1:length(population$info$snp)
   } else{
     subsetting <- TRUE
   }
 
+
+  if(length(array)>0){
+    temp1 <- which(population$info$array.name == array)
+    if(length(temp1)==1){
+      array <- temp1
+    }
+    if(!is.numeric(array)){
+      stop("Input for array can not be assigned! Check your input!")
+    }
+  }
+
   if(population$info$miraculix){
     if (requireNamespace("miraculix", quietly = TRUE)) {
       codeOriginsU <- miraculix::codeOrigins
@@ -70,15 +84,21 @@ get.geno <- function(population, database=NULL, gen=NULL, cohorts=NULL, chromoso
   }
 
 
-  database <- get.database(population, gen, database, cohorts)
+  database <- get.database(population, gen, database, cohorts, id = id)
 
   start.chromo <- cumsum(c(1,population$info$snp)[-(length(population$info$snp)+1)])
   end.chromo <- population$info$cumsnp
 
   relevant.snps <- NULL
-  for(index in chromosomen){
-    relevant.snps <- c(relevant.snps, start.chromo[index]:end.chromo[index])
+
+  if(length(chromosome)==length(start.chromo)){
+    relevant.snps = start.chromo[min(chromosome)]:end.chromo[max(chromosome)]
+  } else{
+    for(index in chromosome){
+      relevant.snps <- c(relevant.snps, start.chromo[index]:end.chromo[index])
+    }
   }
+
   nsnp <- length(relevant.snps)
 
   titel <- t(population$info$snp.base[,relevant.snps])
@@ -106,7 +126,7 @@ get.geno <- function(population, database=NULL, gen=NULL, cohorts=NULL, chromoso
       if(!population$info$miraculix){
         rindex <- 1
         for(index in animals[3]:animals[4]){
-          data[, before + rindex] <- colSums(compute.snps(population,animals[1], animals[2],index, decodeOriginsU=decodeOriginsU))[relevant.snps]
+          data[, before + rindex] <- colSums(computing.snps(population,animals[1], animals[2],index, decodeOriginsU=decodeOriginsU))[relevant.snps]
           rindex <- rindex + 1
         }
       }
@@ -114,14 +134,27 @@ get.geno <- function(population, database=NULL, gen=NULL, cohorts=NULL, chromoso
     before <- before + nanimals
   }
 
+  rownames(data) <- population$info$snp.name[relevant.snps]
+
+
   if(non.genotyped.as.missing){
     is_genotyped <- get.genotyped.snp(population, database = database)[relevant.snps,]
-    if(sum(!is_genotyped)>0){
-      data[!is_genotyped] <- NA
+  } else{
+    is_genotyped = TRUE
+  }
+  if(length(array)>0){
+    if(!non.genotyped.as.missing){
+      is_genotyped <- matrix(TRUE, nrow=nrow(data), ncol=ncol(data))
     }
+    is_genotyped[!population$info$array.markers[[array]], ] <- FALSE
+  }
+  if(sum(!is_genotyped)>0){
+    data[!is_genotyped] <- NA
+  }
+  if(remove.missing && sum(is.na(data[,1]))>0){
+    data <- data[rowMeans(is.na(data))<1,,drop=FALSE]
   }
 
-  rownames(data) <- population$info$snp.name[relevant.snps]
 
   if(use.id){
     colnames(data) <- get.id(population, database = database)
