@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,16 +26,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param database Groups of individuals to consider for the export
 #' @param gen Quick-insert for database (vector of all generations to export)
 #' @param cohorts Quick-insert for database (vector of names of cohorts to export)
-#' @param use.id Set to TRUE to use MoBPS ids instead of Sex_Nr_Gen based names
+#' @param id Individual IDs to search/collect in the database
+#' @param use.id Set to TRUE to use MoBPS ids instead of Sex_Nr_Gen based names (default: TRUE)
+#' @param keep.order To not change order of individuals when ids are provided (default: FALSE)
 #' @examples
 #' data(ex_pop)
 #' get.id(ex_pop, gen=2)
 #' @return Individual ID for in gen/database/cohorts selected individuals
 #' @export
 
-get.id <- function(population, database=NULL, gen=NULL, cohorts=NULL, use.id=FALSE){
+get.id <- function(population, database=NULL, gen=NULL, cohorts=NULL, id = NULL, use.id=TRUE, keep.order = FALSE){
 
-  database <- get.database(population, gen, database, cohorts)
+  database <- get.database(population, gen, database, cohorts, id = id, keep.order = keep.order)
 
   n.animals <- sum(database[,4] - database[,3] +1)
   data <- rep(0, n.animals)
@@ -51,7 +53,7 @@ get.id <- function(population, database=NULL, gen=NULL, cohorts=NULL, use.id=FAL
     }
   }
   if(use.id){
-    names(data) <- get.id(population, database = database)
+    names(data) <- data
   } else{
     names(data) <- names
   }
