@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,15 +26,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param database Groups of individuals to consider for the export
 #' @param gen Quick-insert for database (vector of all generations to export)
 #' @param cohorts Quick-insert for database (vector of names of cohorts to export)
+#' @param id Individual IDs to search/collect in the database
+#' @param use.id Set to TRUE to use MoBPS ids instead of Sex_Nr_Gen based names (default: TRUE)
 #' @examples
 #' data(ex_pop)
 #' get.pheno.off.count(ex_pop, gen=2)
 #' @return Number of offspring with phenotypes for in gen/database/cohorts selected individuals
 #' @export
 
-get.pheno.off.count <- function(population, database=NULL, gen=NULL, cohorts=NULL){
+get.pheno.off.count <- function(population, database=NULL, gen=NULL, cohorts=NULL, id = NULL, use.id = TRUE){
 
-  database <- get.database(population, gen, database, cohorts)
+  database <- get.database(population, gen, database, cohorts, id = id)
 
   n.animals <- sum(database[,4] - database[,3] +1)
   data <- matrix(0, ncol=n.animals, nrow=population$info$bv.nr)
@@ -51,7 +53,12 @@ get.pheno.off.count <- function(population, database=NULL, gen=NULL, cohorts=NUL
 
   }
   row_names <- paste("Trait", 1:population$info$bv.nr)
-  colnames(data) <- names
+
+  if(use.id){
+    colnames(data) <- get.id(population, database = database)
+  } else{
+    colnames(data) <- names
+  }
   rownames(data) <- row_names
   return(data)
 }
