@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,9 +22,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' Miesenberger Index
 #'
 #' Function to selection index weights according to Miesenberger 1997
-#' @param V1 Inverted phenotypic covarianz matrix
-#' @param V Phenotypic covarianz matrix
-#' @param G Genomic covarianz matrix
+#' @param V1 Inverted phenotypic covariance matrix
+#' @param V Phenotypic covariance matrix
+#' @param G Genomic covariance matrix
 #' @param RG Genomic correlation matrix
 #' @param r reliability for the breeding value estimation
 #' @param w relative weighting of each trait (per genetic SD)
