@@ -1,8 +1,8 @@
 '#
   Authors
-Torsten Pook, torsten.pook@uni-goettingen.de
+Torsten Pook, torsten.pook@wur.nl
 
-Copyright (C) 2017 -- 2020  Torsten Pook
+Copyright (C) 2017 -- 2025  Torsten Pook
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,6 +25,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param pedigree Pedigree-file (matrix with 3 columns (Individual ID, Father ID, Mother ID), optional forth columns with earliest generations to generate an individual)
 #' @param keep.ids Set to TRUE to keep the IDs from the pedigree-file instead of the default MoBPS ids
 #' @param plot Set to FALSE to not generate an overview of inbreeding and number of individuals over time
+#' @param halffounder Set to FALSE to make individuals with only one known parent founders (default: TRUE; will additional generate a missing unrelated parent)
+#' @param progress.bar Set to FALSE to not use progress bars in the simulation (Keep log-files lean!)
 #' @param dataset SNP dataset, use "random", "allhetero" "all0" when generating a dataset via nsnp,nindi
 #' @param nsnp number of markers to generate in a random dataset
 #' @param freq frequency of allele 1 when randomly generating a dataset
@@ -35,7 +37,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param length.behind Length after the last SNP of the dataset (default: 5)
 #' @param snps.equidistant Use equidistant markers (computationally faster! ; default: TRUE)
 #' @param snp.position Location of each marker on the genetic map
-#' @param change.order If TRUE sort markers according to given marker positions
 #' @param bv.total Number of traits (If more than traits via real.bv.X use traits with no directly underlying QTL)
 #' @param polygenic.variance Genetic variance of traits with no underlying QTL
 #' @param bit.storing Set to TRUE if the MoBPS (not-miraculix! bit-storing is used)
@@ -54,7 +55,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param exclude.snps Marker were no QTL are simulated on
 #' @param replace.real.bv If TRUE delete the simulated traits added before
 #' @param shuffle.traits Combine different traits into a joined trait
-#' @param shuffle.cor Target Correlation between shuffeled traits
+#' @param shuffle.cor Target Correlation between shuffled traits
 #' @param real.bv.add Single Marker effects
 #' @param real.bv.mult Two Marker effects
 #' @param real.bv.dice Multi-marker effects
@@ -71,7 +72,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param template.chip Import genetic map and chip from a species ("cattle", "chicken", "pig")
 #' @param vcf Path to a vcf-file used as input genotypes (correct haplotype phase is assumed!)
 #' @param vcf.maxsnp Maximum number of SNPs to include in the genotype file (default: Inf)
-#' @param chr.nr Vector containing the assosiated chromosome for each marker (default: all on the same)
+#' @param chr.nr Vector containing the associated chromosome for each marker (default: all on the same)
 #' @param bp Vector containing the physical position (bp) for each marker (default: 1,2,3...)
 #' @param bpcm.conversion Convert physical position (bp) into a cM position (default: 0 - not done)
 #' @param snp.name Vector containing the name of each marker (default ChrXSNPY - XY chosen accordingly)
@@ -94,6 +95,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' @param is.maternal Vector coding if a trait is caused by a maternal effect (Default: all FALSE)
 #' @param is.paternal Vector coding if a trait is caused by a paternal effect (Default: all FALSE)
 #' @param enter.bv Internal parameter
+#' @param output.extended Set TRUE to export more information about the population list
+#' @param trait.cor Target correlation between QTL-based traits (underlying true genomic values)
+#' @param trait.cor.include Vector of traits to be included in the modelling of corrlated traits (default: all - needs to match with trait.cor)
 #' @examples
 #' pedigree <- matrix(c(1,0,0,
 #' 2,0,0,
@@ -104,60 +108,65 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #' 7,1,3,
 #' 8,4,6,
 #' 9,4,7), ncol=3, byrow=TRUE)
-#' population <- pedigree.simulation(pedigree, nsnp=1000)
+#' population = pedigree.simulation(pedigree, nsnp = 100)
 #' @return Population-list
 #' @export
 
 
 
 pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
-                              dataset=NULL, vcf=NULL, chr.nr=NULL, bp=NULL, snp.name=NULL, hom0=NULL, hom1=NULL,
-                                               bpcm.conversion=0,
-                                               nsnp=0, freq="beta", sex.s="fixed",
-                                               chromosome.length=NULL,length.before=5, length.behind=5,
-                                               real.bv.add=NULL, real.bv.mult=NULL, real.bv.dice=NULL, snps.equidistant=NULL,
-                                               change.order=FALSE, bv.total=0, polygenic.variance=100,
-                                               bve.mult.factor=NULL, bve.poly.factor=NULL,
-                                               base.bv=NULL, add.chromosome.ends=TRUE,
-                                               new.phenotype.correlation=NULL,
-                                               new.residual.correlation = NULL,
-                                               new.breeding.correlation=NULL,
-                                               add.architecture=NULL, snp.position=NULL,
-                                               position.scaling=FALSE,
-                                               bit.storing=FALSE,
-                                               nbits=30, randomSeed=NULL,
-                                               miraculix=TRUE,
-                                               miraculix.dataset=TRUE,
-                                               n.additive=0,
-                                               n.dominant=0,
-                                               n.qualitative=0,
-                                               n.quantitative=0,
-                                               var.additive.l=NULL,
-                                               var.dominant.l=NULL,
-                                               var.qualitative.l=NULL,
-                                               var.quantitative.l=NULL,
-                                               exclude.snps=NULL,
-                                               replace.real.bv=FALSE,
-                                               shuffle.traits=NULL,
-                                               shuffle.cor=NULL,
-                                               skip.rest=FALSE,
-                                               enter.bv=TRUE,
-                                               name.cohort=NULL,
-                                               template.chip=NULL,
-                                               beta.shape1=1,
-                                               beta.shape2=1,
-                                               time.point=0,
-                                               creating.type=0,
-                                               trait.name=NULL,
-                                               share.genotyped=1,
-                                               genotyped.s=NULL,
-                                               map=NULL,
-                                               remove.invalid.qtl=TRUE,
-                                               verbose=TRUE,
-                                               bv.standard=FALSE,
-                                               mean.target=NULL,
-                                               var.target=NULL,
-                                               is.maternal = NULL, is.paternal = NULL, vcf.maxsnp=Inf){
+                                dataset=NULL, vcf=NULL, chr.nr=NULL, bp=NULL, snp.name=NULL, hom0=NULL, hom1=NULL,
+                                bpcm.conversion=0,
+                                nsnp=0, freq="beta", sex.s="fixed",
+                                chromosome.length=NULL,length.before=5, length.behind=5,
+                                real.bv.add=NULL, real.bv.mult=NULL, real.bv.dice=NULL, snps.equidistant=NULL,
+                                bv.total=0, polygenic.variance=100,
+                                bve.mult.factor=NULL, bve.poly.factor=NULL,
+                                base.bv=NULL, add.chromosome.ends=TRUE,
+                                new.phenotype.correlation=NULL,
+                                new.residual.correlation = NULL,
+                                new.breeding.correlation=NULL,
+                                add.architecture=NULL, snp.position=NULL,
+                                position.scaling=FALSE,
+                                bit.storing=FALSE,
+                                nbits=30, randomSeed=NULL,
+                                miraculix=TRUE,
+                                miraculix.dataset=TRUE,
+                                n.additive=0,
+                                n.dominant=0,
+                                n.qualitative=0,
+                                n.quantitative=0,
+                                var.additive.l=NULL,
+                                var.dominant.l=NULL,
+                                var.qualitative.l=NULL,
+                                var.quantitative.l=NULL,
+                                exclude.snps=NULL,
+                                replace.real.bv=FALSE,
+                                shuffle.traits=NULL,
+                                shuffle.cor=NULL,
+                                skip.rest=FALSE,
+                                enter.bv=TRUE,
+                                name.cohort=NULL,
+                                template.chip=NULL,
+                                beta.shape1=1,
+                                beta.shape2=1,
+                                time.point=0,
+                                creating.type=0,
+                                trait.name=NULL,
+                                trait.cor = NULL,
+                                trait.cor.include = NULL,
+                                share.genotyped=1,
+                                genotyped.s=NULL,
+                                map=NULL,
+                                remove.invalid.qtl=TRUE,
+                                verbose=TRUE,
+                                bv.standard=FALSE,
+                                mean.target=NULL,
+                                var.target=NULL,
+                                progress.bar = TRUE,
+                                is.maternal = NULL, is.paternal = NULL, vcf.maxsnp=Inf,
+                                halffounder = TRUE,
+                                output.extended = FALSE){
 
   while(ncol(pedigree)<5){
     pedigree <- cbind(pedigree, NA)
@@ -171,6 +180,27 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
   }
 
 
+
+
+  ## adding individuals missing from the pedigree
+  adds = setdiff(pedigree[,2], pedigree[,1])
+  adds = adds[adds != 0]
+
+  if(length(adds) > 0){
+    pedigree = rbind(cbind(adds, 0,0,0,1), pedigree)
+    if(verbose) cat(paste0(length(adds), " male founders added that were sire of pedigree-animals.\n"))
+
+  }
+
+
+  adds = setdiff(pedigree[,3], pedigree[,1])
+  adds = adds[adds != 0]
+  if(length(adds)>0){
+    pedigree = rbind(cbind(adds, 0,0,0,2), pedigree)
+    if(verbose) cat(paste0(length(adds), " female founders added that were dams of pedigree-animals.\n"))
+  }
+
+
   copies <- duplicated(pedigree[,1])
   if(sum(copies)>0){
     pedigree <- pedigree[!copies,]
@@ -179,10 +209,31 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
   n <- nrow(pedigree)
   is_founder <- rep(FALSE, n)
 
-  for(index in 1:n){
-    if(sum(pedigree[index,2]==pedigree[,1])==0 || sum(pedigree[index,3]==pedigree[,1])==0){
-      is_founder[index] <- TRUE
+  is_in1 = !(pedigree[,2] %in% pedigree[,1])
+  is_in2 = !(pedigree[,3] %in% pedigree[,1])
+
+  is_founder = (is_in1 | is_in2)
+  if(halffounder){
+    is_founder = (is_in1 & is_in2)
+    is_halffounder = xor(is_in1 ,is_in2)
+
+    if(sum(is_halffounder)>0){
+
+      add_animal = max(as.numeric(pedigree), na.rm = TRUE) + 1:sum(is_halffounder)
+      pedigree[is_halffounder,2:3][pedigree[is_halffounder,2:3]==0] = add_animal
+      pedigree = rbind(cbind(add_animal,0,0,0,1), pedigree)
+
+      if(verbose) cat(paste0(length(add_animal), " individuals add for simulation of half-founders.\n"))
+      n <- nrow(pedigree)
+      is_founder <- rep(FALSE, n)
+
+      is_in1 = !(pedigree[,2] %in% pedigree[,1])
+      is_in2 = !(pedigree[,3] %in% pedigree[,1])
+
+      is_founder = (is_in1 | is_in2)
+
     }
+
   }
 
   avail <- which(is_founder)
@@ -218,32 +269,58 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
     indexf <- 1
     indexm <- 1
 
-    for(index in (1:n)[-avail]){
-      a <- which(pedigree[index,2]==p_there[,1])
-      b <- which(pedigree[index,3]==p_there[,1])
-      if(length(a)==1 && length(b)==1 && pedigree[index,4]<=(temp1+empty)){
-        poss[index] <- TRUE
-        fixed_temp[index2,] <- c(p_there1[a,], p_there1[b,], pedigree[index,5]-1)
-        if(pedigree[index,5]==1){
-          pedigree_position[index,1:3] <- c(temp1, 1, indexm)
-          indexm <- indexm + 1
-        } else{
-          pedigree_position[index,1:3] <- c(temp1, 2, indexf)
-          indexf <- indexf +1
+    cand = (1:n)[-avail]
+
+    is_in1 = (pedigree[cand,2] %in% p_there[,1])
+    is_in2 = (pedigree[cand,3] %in% p_there[,1])
+
+    cand = cand[is_in1 & is_in2]
+
+    needed = p_there[,1] %in%  c(pedigree[cand,2], pedigree[cand,3])
+    p_there = p_there[needed,,drop = FALSE]
+    p_there1 = p_there1[needed,,drop = FALSE]
+
+
+    for(index5 in 1:ceiling(length(cand)/1000)){
+
+      tmp1 = ((index5-1)*1000+1):min(length(cand), ((index5)*1000))
+      cand_temp = cand[tmp1]
+
+      needed = p_there[,1] %in%  c(pedigree[cand_temp,2], pedigree[cand_temp,3])
+      p_there_tmp = p_there[needed,,drop = FALSE]
+      p_there1_tmp = p_there1[needed,,drop = FALSE]
+
+      for(index in cand_temp){
+        a <- which(pedigree[index,2]==p_there_tmp[,1])
+        b <- which(pedigree[index,3]==p_there_tmp[,1])
+        if(length(a)==1 && length(b)==1 && pedigree[index,4]<=(temp1+empty)){
+          poss[index] <- TRUE
+          fixed_temp[index2,] <- c(p_there1_tmp[a,], p_there1_tmp[b,], pedigree[index,5]-1)
+          if(pedigree[index,5]==1){
+            pedigree_position[index,1:3] <- c(temp1, 1, indexm)
+            indexm <- indexm + 1
+          } else{
+            pedigree_position[index,1:3] <- c(temp1, 2, indexf)
+            indexf <- indexf +1
+          }
+
+
+          index2 <- index2 +1
         }
-
-
-        index2 <- index2 +1
       }
+
     }
-    if(verbose) cat(paste0("Generation ",temp1,": ", index2-1, " individuals. (", sum(!there), " individuals remain)\n"))
+
+
+
+    if(verbose) cat(paste0("Generation ",temp1,": ", index2-1, " individuals. (", sum(!there) - index2 + 1, " individuals remain)\n"))
     if(verbose) cat(paste0("of which ", indexm-1, "/", indexf-1, " are male / female.\n"))
 
     if(index2>1){
       fixed_breeding[[temp1]] <- fixed_temp[1:(index2-1),,drop=FALSE]
     } else{
-     temp1 <- temp1 -1
-     empty <- empty +1
+      temp1 <- temp1 -1
+      empty <- empty +1
     }
 
     avail <- c(avail, which(poss))
@@ -259,7 +336,7 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
                                  length.behind=length.behind,
                                  real.bv.add=real.bv.add, real.bv.mult=real.bv.mult,
                                  real.bv.dice=real.bv.dice, snps.equidistant=snps.equidistant,
-                                 change.order=change.order, bv.total=bv.total, polygenic.variance=polygenic.variance,
+                                 bv.total=bv.total, polygenic.variance=polygenic.variance,
                                  bve.mult.factor=bve.mult.factor, bve.poly.factor=bve.poly.factor,
                                  base.bv=base.bv, add.chromosome.ends=add.chromosome.ends,
                                  new.phenotype.correlation=new.phenotype.correlation,
@@ -283,6 +360,8 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
                                  replace.real.bv=replace.real.bv,
                                  shuffle.traits=shuffle.traits,
                                  shuffle.cor=shuffle.cor,
+                                 trait.cor = trait.cor,
+                                 trait.cor.include = trait.cor.include,
                                  skip.rest=skip.rest,
                                  enter.bv=enter.bv,
                                  name.cohort=name.cohort,
@@ -302,7 +381,8 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
                                  var.target=var.target,
                                  is.maternal = is.maternal,
                                  is.paternal = is.paternal,
-                                 vcf.maxsnp=vcf.maxsnp)
+                                 vcf.maxsnp=vcf.maxsnp,
+                                 progress.bar = progress.bar)
 
   for(index in 2:length(fixed_breeding)){
     if(length(fixed_breeding[[index]])>0){
@@ -327,22 +407,17 @@ pedigree.simulation <- function(pedigree, keep.ids=FALSE, plot=TRUE,
   }
 
   if(keep.ids){
-
     for(index in 1:nrow(pedigree_position)){
       activ <- pedigree_position[index,]
       population$breeding[[activ[1]]][[activ[2]+14]][activ[3]] <- pedigree[index,1]
     }
-
     if(prod(is.numeric(pedigree[,1]))==1){
       population$info$next.animal <- max(pedigree[index,1])+1
     }
-
-
   }
 
-  if(TRUE){
-
-    return(list(population, avail, pedigree_position))
+  if(output.extended){
+    return(list(population, avail, pedigree_position, pedigree))
   } else{
     return(population)
   }
